# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package discovery;

use util::Debug;
use util::XJDParser;
use SOAP::Lite;
use strict;

my $mod = "DiscoveryClient";
my $pipe= "$ENV{HPCALE_CLIENT_LOGPATH}/$ENV{HPCALE_CLIENT_LOGFILE}";

sub discover 
{
   my ($workdir, $xjdprefix) = @_;

   Debug::logmsg($mod, $pipe, "Discovering Resources...");
   Debug::logmsg($mod, "STDOUT", "Discovering Resources...");

   unless(contact_svc($workdir, $xjdprefix)){
      Debug::errmsg($mod, $pipe, "Discovery failed");
      Debug::errmsg($mod, "STDERR", "Discovery failed");
      return -1;
   }

   return 1;
}

sub contact_svc 
{
   my ($workdir, $xjdprefix) = @_;
   
   # Set client information
   $ENV{HTTPS_CERT_FILE} = $ENV{HPCALE_CLIENT_SSLCERT};
   $ENV{HTTPS_KEY_FILE}  = $ENV{HPCALE_CLIENT_SSLKEY};
   my $client = $ENV{HPCALE_CLIENT_HOST};
   my $client_port = $ENV{HPCALE_CLIENT_PORT};
   my $clientdaemon_module = "FrontHandler";

   # set server information
   my $server = $ENV{HPCALE_SERVER_HOST};
   my $port = $ENV{HPCALE_SERVER_PORT};
   my $ws_uri = $ENV{HPCALE_SVC};

   # set service information
   my $svcholder = "svc_discover_rc.cgi";

   # set client soap daemon information
   my $clienturi = "http://$client/$clientdaemon_module";
   my $clientproxy = "http://$client:$client_port";

   # parse high-level XJD file
   my $highXJD = new XJDParser("$workdir/$xjdprefix.xjd");

   # ask discovery
   Debug::logmsg($mod, $pipe, "Contacting $server with port $port");
   Debug::logmsg($mod, "STDOUT", "Contacting $server with port $port");

   my $websvc = SOAP::Lite
      -> uri("http://$server/$ws_uri")
      -> proxy("https://$server:$port/cgi-bin/HPCALE/$svcholder");
   my $result = $websvc->discoverResource($highXJD, 
                                          $clienturi,
                                          $clientproxy);

   unless ($result->fault){
      my $retval = $result -> result();
      if($retval < 0){
         Debug::errmsg($mod, $pipe, "DiscoveryService failed to start.");
         Debug::errmsg($mod, "STDERR", "DiscoveryService failed to start.");
         return 0;
      }
      else{
         Debug::logmsg($mod, $pipe, "DiscoveryService started.");
         Debug::logmsg($mod, "STDOUT", "DiscoveryService started.");
         return 1;
      }
   }
   else{
      my $err = join ', ', $result->faultcode, 
                           $result->faultstring, 
                           $result->faultdetail;
      Debug::errmsg($mod, $pipe, "Webservice fault : $err");
      Debug::errmsg($mod, "STDERR", "Webservice fault : $err");
      return 0;
   }
}

1;
