# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package execution;

use util::Debug;
use util::XJDParser;
use SOAP::Lite;
use strict;

my $mod = "ExecutionClient";
my $pipe= "$ENV{HPCALE_CLIENT_LOGPATH}/$ENV{HPCALE_CLIENT_LOGFILE}";

sub execute {
   my ($workdir, $xjdprefix, $jobid) = @_;

   Debug::logmsg($mod, $pipe, "Executing components...");
   Debug::logmsg($mod, "STDOUT", "Executing components...");

   unless(contact_svc($workdir, $xjdprefix, $jobid)){
      Debug::errmsg($mod, $pipe, "Execution failed");
      Debug::errmsg($mod, "STDERR", "Execution failed");
      return -1;
   }

   return 1;
}

sub contact_svc {
   my ($workdir, $xjdprefix, $jobid) = @_;
   
   # Set client information
   $ENV{HTTPS_CERT_FILE} = $ENV{HPCALE_CLIENT_SSLCERT};
   $ENV{HTTPS_KEY_FILE}  = $ENV{HPCALE_CLIENT_SSLKEY};
   my $client = $ENV{HPCALE_CLIENT_HOST};
   my $client_port = $ENV{HPCALE_CLIENT_PORT};
   my $clientdaemon_module = "FrontHandler";

   # set server information
   my $server = $ENV{HPCALE_SERVER_HOST};
   my $port = $ENV{HPCALE_SERVER_PORT};
   my $ws_uri = $ENV{HPCALE_SVC};
 
   # set service information
   my $svcholder = "svc_execute_comp.cgi";

   # set client soap daemon information
   my $clienturi = "http://$client/$clientdaemon_module";
   my $clientproxy = "http://$client:$client_port";

   # parse low-level XJD
   my $lowXJD = new XJDParser("$workdir/$xjdprefix-low.xjd");
   
   # ask execution 
   Debug::logmsg($mod, $pipe, "Contacting $server with port $port");
   Debug::logmsg($mod, "STDOUT", "Contacting $server with port $port");

   my $websvc = SOAP::Lite
      -> uri("http://$server/$ws_uri")
      -> proxy("https://$server:$port/cgi-bin/HPCALE/$svcholder");

   my $user = $ENV{USER};
   my $host = $ENV{HOST};
   my $result = $websvc->executeComponents($user, $host, $jobid, $lowXJD, $xjdprefix,
                                           $clienturi, 
                                           $clientproxy);

   unless ($result->fault){
      my $retval = $result -> result();
      if($retval < 0){
         Debug::errmsg($mod, $pipe, "Component execution failed to start.");
         Debug::errmsg($mod, "STDERR", "Component execution failed to start.");
         return 0;
      } 
      else{
         Debug::logmsg($mod, $pipe, "Component execution started.");
         Debug::logmsg($mod, "STDOUT", "Component execution started.");
         return 1;
      }
   }
   else{
      my $err= join ', ', $result->faultcode,
                          $result->faultstring, 
                          $result->faultdetail;
      Debug::errmsg($mod, $pipe, "Webservice fault : $err");
      Debug::errmsg($mod, "STDERR", "Webservice fault : $err");
      return 0;
   }
}

1;
