# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package rtenv;

use util::Debug;
use util::XJDParser;
use SOAP::Lite;
use Socket;
use strict;

my $mod = "RTEnvClient";
my $pipe= "$ENV{HPCALE_CLIENT_LOGPATH}/$ENV{HPCALE_CLIENT_LOGFILE}";

sub setup {
   my ($workdir, $xjdprefix, $jobid) = @_;

   Debug::logmsg($mod, $pipe, "RuntimeEnv setup...");
   Debug::logmsg($mod, "STDOUT", "RuntimeEnv setup...");

   unless(contact_svc($workdir, $xjdprefix, $jobid)){
      Debug::errmsg($mod, $pipe, "RuntimeEnv setup failed");
      Debug::errmsg($mod, "STDERR", "RuntimeEnv setup failed");
      return -1;
   }

   return 1;
}

sub contact_svc {
   my ($workdir, $xjdprefix, $jobid) = @_;
   
   # Set client information
   $ENV{HTTPS_CERT_FILE} = $ENV{HPCALE_CLIENT_SSLCERT};
   $ENV{HTTPS_KEY_FILE}  = $ENV{HPCALE_CLIENT_SSLKEY};
   my $client = $ENV{HPCALE_CLIENT_HOST};
   my $client_port = $ENV{HPCALE_CLIENT_PORT};
   my $clientdaemon_module = "FrontHandler";

   # set server information
   my $server = $ENV{HPCALE_SERVER_HOST};
   my $port = $ENV{HPCALE_SERVER_PORT};
   my $ws_uri = $ENV{HPCALE_SVC};
 
   # set service information
   my $svcholder = "svc_runtime_rc.cgi";

   # set client soap daemon information
   my $clienturi = "http://$client/$clientdaemon_module";
   my $clientproxy = "http://$client:$client_port";

   my $lowXJD = new XJDParser("$workdir/$xjdprefix-low.xjd");
   
   # ask runtime environment preparation 
   Debug::logmsg($mod, $pipe, "Contacting $server with port $port");
   Debug::logmsg($mod, "STDOUT", "Contacting $server with port $port");

   my $websvc = SOAP::Lite
      -> uri("http://$server/$ws_uri")
      -> proxy("https://$server:$port/cgi-bin/HPCALE/$svcholder");

   my $user = $ENV{USER};
   my $host = $ENV{HOST};
   my $result = $websvc->prepareEnv($user, $host, $jobid, $lowXJD,
                                    $clienturi,
                                    $clientproxy);

   unless ($result->fault){
      my $retval = $result -> result();
      if($retval < 0){
         Debug::errmsg($mod, $pipe, "Preparing runtime environments failed to start.");
         Debug::errmsg($mod, "STDERR", "Preparing runtime environments failed to start.");
         return 0;
      } 
      else{
         Debug::logmsg($mod, $pipe, "Preparing runtime environments started.");
         Debug::logmsg($mod, "STDOUT", "Preparing runtime environments started.");
         return 1;
      }
   }
   else{
      my $err= join ', ', $result->faultcode,
                          $result->faultstring, 
                          $result->faultdetail;
      Debug::errmsg($mod, $pipe, "Webservice fault : $err");
      Debug::errmsg($mod, "STDERR", "Webservice fault : $err");
      return 0;
   }
}
 
1;
