# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package termination;

use util::Debug;
use util::XJDParser;
use SOAP::Lite;
use strict;

my $mod = "JobTermination";

sub stopjob {
   my ($jobid) = @_;

   Debug::logmsg($mod, "STDOUT", "Terminating job $jobid");

   unless(contact_svc($jobid)){
      Debug::logmsg($mod, "STDOUT", "Job termination failed");
      return -1;
   }
}

sub contact_svc {
   my ($jobid) = @_;
   
   # Set client information
   $ENV{HTTPS_CERT_FILE} = $ENV{HPCALE_CLIENT_SSLCERT};
   $ENV{HTTPS_KEY_FILE}  = $ENV{HPCALE_CLIENT_SSLKEY};
   my $client = $ENV{HPCALE_CLIENT_HOST};
   my $client_port = $ENV{HPCALE_CLIENT_PORT};
   my $clientdaemon_module = "FrontHandler";

   # set server information
   my $server = $ENV{HPCALE_SERVER_HOST};
   my $port = $ENV{HPCALE_SERVER_PORT};
   my $ws_uri = $ENV{HPCALE_SVC};
 
   # set service information
   my $svcholder = "svc_stop_job.cgi";

   # set client soap daemon information
   my $clienturi = "http://$client/$clientdaemon_module";
   my $clientproxy = "http://$client:$client_port";

   # ask termination 
   Debug::logmsg($mod, "STDOUT", "Contacting $server with port $port");

   my $websvc = SOAP::Lite
      -> uri("http://$server/$ws_uri")
      -> proxy("https://$server:$port/cgi-bin/HPCALE/$svcholder");

   my $user = $ENV{USER};
   my $host = $ENV{HOST};
   my $result = $websvc->stopComponents($user, $host, $jobid, 
                                        $clienturi, $clientproxy);

   unless ($result->fault){
      my $retval = $result -> result();
      if($retval < 0){
         Debug::errmsg($mod, "STDERR", "Job termination failed to start.");
         return 0;
      } 
      else{
         Debug::logmsg($mod, "STDOUT", "Job termination started.");
         return 1;
      }
   }
   else{
      my $err= join ', ', $result->faultcode,
                          $result->faultstring, 
                          $result->faultdetail;
      Debug::errmsg($mod, "STDERR", "Webservice fault : $err");
      return 0;
   }
}


1;
