# Written by Il-Chul Yoon <iyoon@cs.umd.edu>

package validator;

use util::Debug;
use vars qw/ %opt /;
use Getopt::Std;
use strict;

my $mod   = "ProgInputValidator";
my $pipe= "$ENV{HPCALE_LOGPATH}/$ENV{HPCALE_LOGFILE}";

my $errmsg01 = "HPCALE requires XJDFILE as argument.";
my $errmsg02 = "XJD file name is not valid.";
my $errmsg03 = "job identifier is necessary";

sub checkoption_x 
{
   getopts( "hx:", \%opt);

   if ($opt{x}) {
      Debug::dbgmsg(2, $mod, $pipe, "$opt{x} is given as XJD file.");
      return $opt{x};
   }
   else{
      Debug::logmsg($mod, $pipe, $errmsg01);
      exit;
   }
}

sub checkoption_j 
{
   getopts( "j:", \%opt);

   if ($opt{j}) {
      Debug::dbgmsg(2, $mod, $pipe, "$opt{j} is given as job identifier.");
      return $opt{j};
   }
   else{
      Debug::logmsg($mod, $pipe, $errmsg03);
      exit;
   }
}

sub validate_x 
{
   my $xjdfile = shift;

   $xjdfile =~ /(.+)\/(.+)\.xjd/;

   if(length($1) == 0 || length($2) == 0){
      Debug::logmsg($mod, $pipe, $errmsg02);
   }
   my $workdir = $1;
   my $xjdprefix = $2;

   unless ($workdir =~ /^\//){
      my $pwd = `pwd`; 
      chomp $pwd;

      $workdir = "$pwd/$workdir";
      Debug::dbgmsg(1, $mod, $pipe, "Workdir: $workdir");
      Debug::dbgmsg(1, $mod, $pipe, "XJDfile prefix: $xjdprefix");
   }   

   return $workdir, $xjdprefix;
}

sub validate_j
{
   my $jobid = shift;

   if($jobid =~ /\d+/){
      return $jobid;
   }
   else{
      Debug::errmsg(1, $mod, $pipe, "Wrong job identifier format");
      exit;
   }
}

sub usage {
   print "===================================================\n";
   print "Usage:\n";
   print "  perl launch.pl -x XJDFILE\n";
   print "\n";
   print "     -h : show this help\n";
   print "     -v : show version\n";
   print "===================================================\n";
}

sub usage2 {
   print "===================================================\n";
   print "Usage:\n";
   print "  perl stopjob.pl -x XJDFILE -j jobid\n";
   print "\n";
   print "     -h : show this help\n";
   print "===================================================\n";
}

sub printVersion {
   print "HPCALE Client Version 1.0\n";
}

1;

