# Written by Il-Chul Yoon <iyoon@cs.umd.edu>


package FileHandler;

use util::Debug;
use Socket;
use File::Path;
use strict;

my $mod = "Clientd-FH";
my $pipe= "$ENV{HPCALE_CLIENT_LOGPATH}/$ENV{HPCALE_CLIENT_LOGFILE}";

sub contact_ftpd 
{
   my ($host, $port, $jobid, $cert) = @_;
   Debug::logmsg($mod, $pipe, "contacting ftpd...");
   
   my $proto  = getprotobyname('tcp');
   my $iaddr = inet_aton($host);
   my $paddr = sockaddr_in($port, $iaddr);

   # create socket
   socket(SOCK, PF_INET, SOCK_STREAM, $proto) or return 0;

   # establish connection 
   connect(SOCK, $paddr) or return 0;

   # check certificate
   &check_cert($jobid,$cert) or return 0;
   &send_int($jobid);

   return 1;
}

sub terminate_connection 
{
   Debug::logmsg($mod, $pipe, "disconnecting ftpd...");
   shutdown (SOCK, 1);
}

sub check_cert 
{
   my $jobid = shift;
   my $cert = shift;
   send_file("localhost", $cert, "HPCALE-Server", "cert-$jobid.pem");

   my $result = '';
   recv(SOCK, $result, 1, 0); # certificate check result
   if($result == 1){
      Debug::logmsg($mod, $pipe, "client certificate is ok");
      return 1;
   }
   else{
      Debug::errmsg($mod, $pipe, "invalid client certificate");
      return 0;
   }
}

sub get_filelist
{
   my ($wd, $specfile) = @_;
   my @files = ();

   open SPEC, "< $wd/$specfile";
   my @lines = <SPEC>;
   close SPEC;

   # assume correct input file format
   foreach(@lines){
      chomp($_);
      $_ = pack_line($_);

      if($_ =~ /^#/){
         # comment
      }
      elsif(length($_) == 0){
         # empty line
      }
      else{
         my @arg = split /,/, $_;
         
         $arg[0] = pack_line($arg[0]);
         $arg[1] = pack_line($arg[1]);

         Debug::dbgmsg(3, $mod, $pipe, "Add File: $arg[0], $arg[1]");
         my @from = split /:/, $arg[0];
         my @to = split /:/, $arg[1];
        
         if(not ($from[1] =~ /^\//)){
            $from[1] = $wd ."/". $from[1];
            Debug::dbgmsg(3, $mod, $pipe, "$from[1]");
         }
         push @files, [$from[0], $from[1], $to[0], $to[1]];
      }
   }

   return @files;
}

sub get_filelist_outfile
{
   my ($wd, $specfile) = @_;
   my @files = ();

   open SPEC, "< $wd/$specfile";
   my @lines = <SPEC>;
   close SPEC;

   # assume correct input file format
   foreach(@lines){
      chomp($_);
      $_ = pack_line($_);

      if($_ =~ /^#/){
         # comment
      }
      elsif(length($_) == 0){
         # empty line
      }
      else{
         my @arg = split /,/, $_;
         
         $arg[0] = pack_line($arg[0]);
         $arg[1] = pack_line($arg[1]);

         Debug::dbgmsg(3, $mod, $pipe, "Add File: $arg[0], $arg[1]");
         my @from = split /:/, $arg[0];
         my @to = split /:/, $arg[1];
        
         if(not ($from[1] =~ /^\//)){
            Debug::dbgmsg(3, $mod, $pipe, "$from[1]");
         }
         push @files, [$from[0], $from[1], $to[0], $to[1]];
      }
   }

   return @files;
}

sub recv_file
{
   my ($fh, $fn, $th, $tn, $wdir) = @_;
   my $data = '';

   # based on these two strings, server will grab the files from resource
   # before transfer to client
   &send_str($fh); # send src. host name
   &send_str($fn); # send src. file name

   my $recv_fn = "";

   if($tn =~ /(.*)\/(.+)/){
      my $path = $1;
      if($tn =~ /^\/.+/){
         Debug::dbgmsg(3, $mod, $pipe, "making absolute dir: $path");
         mkpath(["$path"], 1, 0711);
         $recv_fn = $tn;
      }
      else{
         Debug::dbgmsg(3, $mod, $pipe, "making relative dir: $wdir/$path");
         mkpath(["$wdir/$path"], 1, 0711);

         $recv_fn = "$wdir/$tn";
      }
   }
   else{
      $recv_fn = "$wdir/$tn";
   }
   Debug::logmsg($mod, $pipe, "Filename at Client: $recv_fn");

   # recv one integer to know server copied the output file from resources
   recv(SOCK, $data, 1, 0);
   if($data == 0){
      Debug::errmsg($mod, $pipe, "server failed to grab $fn from $fh.");
      return -1;
   }

   my $size = &recv_int();

   # no need to recv. just make empty file
   if($size == 0){
      open TORECV, "> $recv_fn";
      close TORECV;

      # last ack for each file
      recv(SOCK, $data, 1, 0);
      if($data == 0){
         Debug::errmsg($mod, $pipe, "server may think that file transfer failed."); 
         return -1;
      }
      return 1;
   }

   Debug::logmsg($mod, $pipe, "start receiving $recv_fn...");

   my $data = '';
   my $chunksize = 1000;

   Debug::dbgmsg(3, $mod, $pipe, "<<$recv_fn, $size>>");

   my $quot = $size / $chunksize;
   my $remain = $size % $chunksize;

   # start recv
   open TORECV, "> $recv_fn";
   binmode TORECV;

   my $i = 1;
   while ($i <= $quot){
      recv(SOCK, $data, $chunksize, 0);
      if(length($data) < 1000){
         Debug::dbgmsg(3, $mod, $pipe, "data recvd: " . length($data));
         send(SOCK, "0", 0);
         next;
      }
      else{
         send(SOCK, "1", 0);
      }
      
      print TORECV $data;
      $data = '';
      $i++;
   }

   $data = '';

   while(1){
      recv(SOCK, $data, $remain, 0);
      my $a = length($data);
      Debug::dbgmsg(3, $mod, $pipe, "$a <-> $remain");

      if($a == $remain){
         send(SOCK, "1", 0);
         last;
      }
      else{
         send(SOCK, "0", 0);
      }
      
      $data = '';
   }

   print TORECV $data;
   close TORECV;
   # end recv

   # last ack for each file
   recv(SOCK, $data, 1, 0);
   if($data == 0){
      Debug::errmsg($mod, $pipe, "server think file transfer failed."); 
      return -1;
   }

   my $recvdsize = -s $recv_fn;
   return ($size == $recvdsize) ? 1 : 0; 
}

sub send_file 
{
   my ($fh, $fn, $th, $tn) = @_;
   my $data = '';
   my $result = '';

   &send_str($th); # send dest. host name
   &send_str($tn); # send dest. file name

   open TOSEND, "< $fn";
   binmode TOSEND;

   # send file size
   my $size = -s TOSEND;
   Debug::dbgmsg(3, $mod, $pipe, "file($fh:$fn), size: $size");
   &send_int($size);

   recv(SOCK, $result, 1, 0);
   if($result == 0){
      Debug::errmsg($mod, $pipe, "Invalid filename or zero file size.($fh:$fn)");
      close TOSEND;
      return 0;
   }
   else{
      Debug::logmsg($mod, $pipe, "Start to send the file $fn");
   }

   # send file contents
   my $chunksize = 1000;
   my $quot = $size / $chunksize;
   my $remain = $size % $chunksize;
   
   my $i = 1;
   my $sendok = 1;
   while($i <= $quot) {
      if($sendok == 1){
         read(TOSEND, $data, $chunksize);
      }

      my $realsent = send(SOCK, $data, 0);  

      recv(SOCK, $data, 1, 0);
      if($data == 1){
         $sendok = 1;
         $data = ''; 
      }
      else{
         $sendok = 0;
      }
      $i++;
   }
   
   $data = '';
   read(TOSEND, $data, $remain);
   send(SOCK, $data, 0);
   # end of file contents transfer

   close TOSEND;

   # recv file migration result
   recv(SOCK, $result, 1, 0);
   ($result == 1) ? return 1 : return 0;
}

sub send_str
{
   my $str = shift;
   &send_int(length($str));
   send(SOCK, $str, 0); # send str
}

sub send_int
{
   my $int = shift;
   send(SOCK, "$int.", 0); # send integer and end mark

   return 1;
}

sub recv_int
{
   my $char = '';
   my $size = '';

   while(1){
      recv(SOCK, $char, 1, 0);
      if($char eq '.'){
         last;
      }
      $size .= $char;
   }

   return $size;
}

sub pack_line 
{
   my $str = shift;
   chomp $str;
   $str =~ s/^\s+//;
   $str =~ s/\s+$//;
   return $str;
}

1;

