# $Author: iyoon $
# $Date: 2007/01/04 19:42:58 $
# $Revision: 1.2 $
#
# Modified by Il-Chul Yoon <iyoon@cs.umd.edu>
# Extracted from Skoll project at University of Maryland, College Park

eval '(exit $?0)' && eval 'exec perl -S $0 ${1+"$@"}'
    & eval 'exec perl -S $0 $argv:q'
    if 0;

use strict;
use warnings;
use Cwd;

my ($errors, $warnings) = (0, 0);
my $perl_module_lst_file = "./clientpm.lst";

&check_perl_modules ;
&check_env_vars ; 


sub check_env_vars {
   my $chkresult = 1;

   print "\nChecking Environment Variables ...\n";
   my @envvars = qw/ HPCALE_CLIENT_HOST
                     HPCALE_CLIENT_HOME
                     HPCALE_CLIENT_PORT
                     HPCALE_CLIENT_LOGPATH
                     HPCALE_CLIENT_LOGFILE
                     HPCALE_DEBUG_LEVEL
                     HPCALE_CLIENT_SSLCERT
                     HPCALE_CLIENT_SSLKEY
                     HPCALE_SERVER_HOST
                     HPCALE_SERVER_PORT
                     HPCALE_SVC
                     HPCALE_FTPD_HOST
                     HPCALE_FTPD_PORT /;

   foreach(@envvars){
      if(chk_envvar($_) < 0){
         $chkresult = -1;
      }
   }

   if($chkresult < 0){
      print "Missing Environment Variable.\n";
      exit;
   }
   else{
      print "All necessary environment variables are defined.\n";
   }
}

sub chk_envvar {
   my $var = shift;
   if(!defined($ENV{$_})){
      print STDERR "Undefined environment variable: $var\n";
      return -1;
   }
   else{
      print STDERR "  Checking for $var....yes\n";
      return 1;
   }
}

sub check_perl_modules {
    # checking perl modules
    print "\nChecking perl modules ...\n";
    
    unless ($] >= 5.006) {
        die "Sorry, you need at least Perl 5.6\n";
    }

    # Check versions of dependencies.  0 for version = any version acceptible
    # Read the version file
    my $modules = ();
    open(LST, "< $perl_module_lst_file")
            || die("Cannot read \"$perl_module_lst_file\"");
    while(<LST>){
            if (m/^\s*(\S+)\s+(\S+)\s*$/){
               push(@{$modules}, {name=>$1, version=>$2});
            }
    }
    close(LST);
    
    my %missing = ();
    
    # Modules may change $SIG{__DIE__} and $SIG{__WARN__}, so localise them here
    # so that later errors display 'normally'
    foreach my $module (@{$modules}) {
        local $::SIG{__DIE__};
        local $::SIG{__WARN__};
        unless (have_vers($module->{name}, $module->{version})) {
            $missing{$module->{name}} = $module->{version};
            $errors++;
        }
    }
    
    if (%missing) {
        print "\n\n";
        print "HPCALE requires some Perl modules which are either missing from your\n",
        "system, or the version on your system is too old.\n",
        "They can be installed by running (as root) the following:\n";
        foreach my $module (keys %missing) {
            print "   perl -MCPAN -e 'install \"$module\"'\n";
            if ($missing{$module} > 0) {
                print "   Minimum version required: $missing{$module}\n";
            }
        }
        print "\n";
        exit;
    }
    
    print "\nYou have all required Perl modules\n";
    return 1;
}


# vers_cmp is adapted from Sort::Versions 1.3 1996/07/11 13:37:00 kjahds,
# which is not included with Perl by default, hence the need to copy it here.
# Seems silly to require it when this is the only place we need it...
sub vers_cmp {
  if (@_ < 2) { die "not enough parameters for vers_cmp" }
  if (@_ > 2) { die "too many parameters for vers_cmp" }
  my ($a, $b) = @_;
  my (@A) = ($a =~ /(\.|\d+|[^\.\d]+)/g);
  my (@B) = ($b =~ /(\.|\d+|[^\.\d]+)/g);
  my ($A,$B);
  while (@A and @B) {
    $A = shift @A;
    $B = shift @B;
    if ($A eq "." and $B eq ".") {
      next;
    } elsif ( $A eq "." ) {
      return -1;
    } elsif ( $B eq "." ) {
      return 1;
    } elsif ($A =~ /^\d+$/ and $B =~ /^\d+$/) {
      return $A <=> $B if $A <=> $B;
    } else {
      $A = uc $A;
      $B = uc $B;
      return $A cmp $B if $A cmp $B;
    }
  }
  @A <=> @B;
}


# This was originally clipped from the libnet Makefile.PL, adapted here to
# use the above vers_cmp routine for accurate version checking.
sub have_vers {
  my ($pkg, $wanted) = @_;
  my ($msg, $vnum, $vstr);
  no strict 'refs';
  printf("Checking for %15s %-9s ", $pkg, !$wanted?'(any)':"(v$wanted)");

  eval { my $p; ($p = $pkg . ".pm") =~ s!::!/!g; require $p; };

  $vnum = ${"${pkg}::VERSION"} || ${"${pkg}::Version"} || 0;
  $vnum = -1 if $@;

  if ($vnum eq "-1") { # string compare just in case it's non-numeric
    $vstr = "not found";
  }
  elsif (vers_cmp($vnum,"0") > -1) {
    $vstr = "found v$vnum";
  }
  else {
    $vstr = "found unknown version";
  }

  my $vok = (vers_cmp($vnum,$wanted) > -1);
  print ((($vok) ? "ok: " : " "), "$vstr\n");
  return $vok;
}

